#!/usr/bin/perl -w
#
# mp3tovorbis:
#
# This script converts a list of mp3 files or a hole directory containing
# mp3 files to the Ogg/Vorbis audio format, which has serveral advantages
# compared to mp3, which the most important one: It is *free* software.
# 
# Copyright (c) 2001, 2002 Benedikt Meurer <bmeurer@fwdn.de>
# 

use strict;
use MP3::Info;
use Pod::Usage;

# options:
my %options = (
	'remove' => 0,		# if 1, MP3 files will be unlinked after converting
);

# set program name
$0 = "mp3tovorbis";

# parse the command line
ARGS: while (my $arg = shift @ARGV) {
	# -r|--remove
	if ($arg eq '-r' or $arg eq '--remove') {
		$options{'remove'} = 1;

	# -k|--keep
	} elsif ($arg eq '-k' or $arg eq '--keep') {
		$options{'remove'} = 0;

	# -h|--help
	} elsif ($arg eq '-h' or $arg eq '--help') {
		&pod2usage (-exitvalue => 1, -output => \*STDERR);

	# -- stops param parsing
	} elsif ($arg eq '--') {
		last ARGS;

	# unknown arg
	} elsif ($arg =~ /^-/) {
		&pod2usage (-exitvalue => 1, -output => \*STDERR,
					-message => "Unknown option: $arg");

	# anything else is assumed to be a file
	} else {
		unshift @ARGV, $arg;
		last ARGS;
	}
}

# parameters to call mpg123 through exec
my @mpg123 = ('/usr/bin/mpg123', '--quiet', '--stereo',	'-r', '44100', '-s');

# parameters to call oggenc through exex
my @oggenc = ('/usr/bin/oggenc', '--quiet',	'-b', '128', '-M', '192', '-r');

# number of current childs and number of errors occured
my ($childs, $errors) = (0, 0);

# hashes of files currently processed by mpg123/oggenc; hash key is the
# pid of the process
my %ogg_files = ();
my %mp3_files = ();

# get id3 tags from mp3 file or from the filename
sub id3 {
	my $filename = shift;
	my %tags = ();

	# examine filename
	if ($filename =~ /^([^-]+)-([^-]+)\.mp3$/oi) {
		# file is in Format "ARTIST - TITLE.mp3" (hopefully)
		($tags{'ARTIST'}, $tags{'TITLE'}) = ($1, $2);

	} elsif ($filename =~ /^([\w\s\d]+)\.mp3$/io) {
		# treat hole filename (except the trailing .mp3) as title
		$tags{'TITLE'} = $1;
	}

	# read ID3 tag from MP3 file
	my $mp3 = &get_mp3tag ($filename);

	# apply all found tags ...
	foreach (keys %$mp3) {
		$tags{$_} = $mp3->{$_};
	}

	# ... and return them
	return \%tags;
}

# convert a mp3 file to an ogg/vorbis file
sub mp3tovorbis {
	my ($filemp3, $tags) = @_;
	(my $filebase = $filemp3) =~ s/\.mp3$//oi;

	# create unnamed pipe
	pipe (IN, OUT);

	# open error log
	open (ERR, ">&STDERR");

	# fork for mpg123
	if (my $pid = fork ()) {
		$childs++;
		$mp3_files{$pid} = $filemp3;
	} elsif (defined $pid and $pid == 0) {
		open (STDOUT, ">&OUT");
		open (STDERR, ">&ERR");

		exec (@mpg123, '--', "$filemp3")
			or die "Cannot exec mpg123: $!";
	} else {
		die "Cannot fork: $!";
	}

	# fork for oggenc
	if (my $pid = fork ()) {
		$childs++;
		$ogg_files{$pid} = "$filebase.ogg";
	} elsif (defined $pid and $pid == 0) {
		open (STDIN, "<&IN");
		open (STDERR, ">&ERR");


		# append outfile name to the oggenc exec parameters
		push @oggenc, '-o';
		push @oggenc, "$filebase.ogg";

		if ($tags->{'TITLE'}) {
			push @oggenc, '-t';
			push @oggenc, $tags->{'TITLE'};
		}

		if ($tags->{'ARTIST'}) {
			push @oggenc, '-a';
			push @oggenc, $tags->{'ARTIST'};
		}

		if ($tags->{'ALBUM'}) {
			push @oggenc, '-l';
			push @oggenc, $tags->{'ALBUM'};
		}

		if ($tags->{'YEAR'}) {
			push @oggenc, '-c';
			push @oggenc, "date=" . int ($tags->{'YEAR'});
		}

		if ($tags->{'TRACKNUM'}) {
			push @oggenc, '-c';
			push @oggenc, 'tracknumber=' . $tags->{'TRACKNUM'};
		}

		if ($tags->{'COMMENT'}) {
			push @oggenc, '-c';
			push @oggenc, '=' . $tags->{'COMMENT'};
		}

		if ($tags->{'GENRE'}) {
			push @oggenc, '-c';
			push @oggenc, 'genre=' . $tags->{'GENRE'};
		}

		exec (@oggenc, '-')
			or die "Cannot exec oggenc: $!";
	} else {
		die "Cannot fork: $!";
	}

	# close all used file handles
	close IN;
	close OUT;
	close ERR;
}

sub child_died {
	# get pid of finished process
	my $pid = wait ();

	# check if it was an oggenc process ...
	if (defined $ogg_files{$pid}) {
		if ($? == 0) {
			print "File \"$ogg_files{$pid}\" created successfully\n";
		} else {
			# we encountered an error
			$errors++;
			print "Failed to create \"$ogg_files{$pid}\"\n";
		}

	# or an mpg123 process
	} elsif (defined $mp3_files{$pid}) {
		if ($? == 0) {
			# remove old mp3 file
			unlink ($mp3_files{$pid}) if ($options{'remove'});
		} else {
			$errors++;
		}
	
	# if we reach this else block, something went veeeery wrong
	} else {
		die "Something went veeery wrong. Please contact the maintainer!";
	}

	# lower the number of running childs
	$childs--;
}

# register handler for SIG_CHLD
$SIG{CHLD} = 'child_died';

# list of specified files or all MP3 files in the current directory
my @files = (scalar (@ARGV) > 0) ? @ARGV : glob ('*.[Mm][Pp]3');

# convert all files
foreach my $filename (@files) {
	&mp3tovorbis ($filename, &id3 ($filename));

	# wait while more then one child is active
	sleep 5 while ($childs > 1);
}

# wait for all childs to terminate
sleep 1 while ($childs > 0);

exit (($errors > 0) ? 1 : 0);


__END__

=head1 NAME

mp3tovorbis - MP3 to Ogg/Vorbis converter

=head1 SYNOPSIS

mp3tovorbis [options] [file ...]

 Options:
 	-h, --help        brief help message
	-k, --keep        keep mp3 files (default)
	-r, --remove      remove mp3 files after converted to ogg files

=head1 OPTIONS

=over 8

=item B<-h, --help>

Prints a brief help message and exists.

=item B<-k, --keep>

Don't delete MP3 files after converting them to Ogg/Vorbis. This is the
default behaviour.

=item B<-r, --remove>

Remove each MP3 file after it has been converted to an Ogg/Vorbis file.

=back

=head1 DESCRIPTION

B<mp3tovorbis> converts MP3 files to Ogg/Vorbis.

=head1 AUTHOR

Written by Benedikt Meurer <bmeurer@fwdn.de>.

=cut
